angular.module('timeline', ['ngResource', 'chart.js', 'wemServices', 'wemDirectives', 'ui.bootstrap', 'i18n', 'underscore'])
    .directive("timeline", function () {
    return {
        restrict: "E",
        scope: {
            data: "=ngModel",
            colours: "=colours",
            report: "="
        },
        templateUrl: ManagersContext.localTemplatesPath + "/timeline.html",
        controller: ['$scope', "i18nService", "datepickerConfig", function ($scope, i18nService, datepickerConfig) {
            validateDates = function(fromDate, toDate, target) {
                var date;
                if (target == 'toDate') {
                    date = toDate;
                    var min = moment(fromDate);
                    min.add($scope.data.interval.unitcount * $scope.data.interval.minpoints, $scope.data.interval.units);
                    var max = moment(fromDate);
                    max.add($scope.data.interval.unitcount * $scope.data.interval.maxpoints, $scope.data.interval.units);
                }
                if (target == 'fromDate') {
                    date = fromDate;
                    var min = moment(toDate);
                    min.add(- $scope.data.interval.unitcount * $scope.data.interval.maxpoints, $scope.data.interval.units);
                    var max = moment(toDate);
                    max.add(- $scope.data.interval.unitcount * $scope.data.interval.minpoints, $scope.data.interval.units);
                }
                if (date.isAfter(max)) {
                    return max;
                }
                if (date.isBefore(min)) {
                    return min;
                }
            };

            $scope.filterDates = function(target) {
                var filtered = {};
                var fromDateMoment = moment($scope.data.fromDate);
                var toDateMoment = moment($scope.data.toDate);
                var dates = [];

                // Validate date interval
                if (target) {
                    var newToDateMoment = validateDates(fromDateMoment, toDateMoment, target);
                    if (newToDateMoment) {
                        $scope.data[target] = newToDateMoment.toDate();
                    }
                }

                // Fill empty points with 0
                if (fromDateMoment.isBefore(toDateMoment)) {
                    var count = 0;
                    var dif = toDateMoment.diff(fromDateMoment,  $scope.data.interval.units);
                    dates.push(fromDateMoment.format("YYYY-MM-DD"));
                    while (count < dif) {
                        dates.push(fromDateMoment.add($scope.data.interval.unitcount,  $scope.data.interval.units).format("YYYY-MM-DD"));
                        count++;
                    }
                }

                filtered = [];
                angular.forEach($scope.datasets, function(d,i) {
                    filtered[i] = [];
                });

                angular.forEach(dates, function (v, k) {
                    angular.forEach($scope.datasets, function(d,i) {
                        filtered[i].push($scope.datasets[i][v] ? $scope.datasets[i][v] : 0);
                    })
                });
                var labels = _.map(dates, function(f) { return moment(f).format( $scope.data.interval.scaleFormat) });
                $scope.lineChartData = {
                    labels: labels,
                    datasets: filtered,
                    series: $scope.data.series,
                    colours: $scope.colours ? $scope.colours : [ManagersUtils.firstlineChartSettings, ManagersUtils.secondlineChartSettings],
                    options: angular.extend({
                        scaleBeginAtZero: true
                    }, $scope.data.chartOptions)
                };
            };

            $scope.queryData = function () {
                $scope.data.loadData(function(datasets) {
                    $scope.datasets = datasets;
                    $scope.filterDates("fromDate");
                    $scope.watchingDate = true;
                });
            };

            // compute interval
            $scope.data.interval = $scope.data.interval || ManagersUtils.supportedTimelineIntervales["1M"];
            $scope.intervals = ManagersUtils.supportedTimelineIntervales;

            // query
            $scope.queryData();

            // attach listener:
            // reload data
            $scope.watchingDate = false;
            $scope.$watch('data.fromDate', function() {
                if ($scope.watchingDate) {
                    $scope.filterDates('toDate');
                }
            });
            $scope.$watch('data.toDate', function() {
                if ($scope.watchingDate) {
                    $scope.filterDates('fromDate');
                }
            });


            $scope.openDatePopup = function ($event, selector) {
                $event.preventDefault();
                $event.stopPropagation();

                if (selector == 'start') {
                    $scope.startOpened = true;
                    $scope.endOpened = false;
                } else {
                    $scope.endOpened = true;
                    $scope.startOpened = false;
                }
            };

            // datepicker config
            datepickerConfig.startingDay = 1;
            datepickerConfig.minMode = $scope.data.interval.unit;
            datepickerConfig.datepickerMode = $scope.data.interval.unit;
            datepickerConfig.showWeeks = false;

            $scope.currentText = ManagersContext.i18nLabels['serverSettings.manageWem.datepicker.today'];
            $scope.closeText = ManagersContext.i18nLabels['serverSettings.manageWem.datepicker.close'];

            // convert dates when interval change
            $scope.$watch('data.interval', function(newVal, oldVal){
                if(oldVal && newVal && newVal != oldVal) {
                    watchingDate = false;
                    var fromDateMoment = moment($scope.data.fromDate);
                    var toDateMoment = moment($scope.data.toDate);
                    fromDateMoment.date(1);
                    toDateMoment.date(1);
                    var newToDateMoment = validateDates(fromDateMoment, toDateMoment, 'toDate');
                    if (newToDateMoment) {
                        toDateMoment = newToDateMoment;
                    }
                    $scope.data.fromDate = fromDateMoment.toDate();
                    $scope.data.toDate = toDateMoment.toDate();

                    // update datepicker-popup format which is *NOT* the same as moment's format >_<

                    // update datepicker options
                    datepickerConfig.minMode = newVal.unit;
                    datepickerConfig.datepickerMode = newVal.unit;

                    $scope.queryData();
                }
            });
        }]
    };
});

